@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * netlink/route/link/vlan.h		VLAN interface
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -34,6 +28,9 @@ extern int		rtnl_link_is_vlan(struct rtnl_link *);
 extern char *		rtnl_link_vlan_flags2str(int, char *, size_t);
 extern int		rtnl_link_vlan_str2flags(const char *);
 
+extern int		rtnl_link_vlan_set_protocol(struct rtnl_link *link, uint16_t);
+extern int		rtnl_link_vlan_get_protocol(struct rtnl_link *link);
+
 extern int		rtnl_link_vlan_set_id(struct rtnl_link *, uint16_t);
 extern int		rtnl_link_vlan_get_id(struct rtnl_link *);
 
