@@ -3,13 +3,18 @@
  * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
  */
 
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
+#include <linux/socket.h>
+
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/addr.h>
 #include <netlink/attr.h>
 #include <netlink/msg.h>
-#include <linux/socket.h>
+
+#include "nl-priv-dynamic-core/nl-core.h"
+#include "nl-aux-core/nl-core.h"
 
 /**
  * @ingroup msg
@@ -124,7 +129,7 @@ void *nla_data(const struct nlattr *nla)
  */
 int nla_len(const struct nlattr *nla)
 {
-	return nla->nla_len - NLA_HDRLEN;
+	return _nla_len(nla);
 }
 
 /**
@@ -141,6 +146,8 @@ int nla_len(const struct nlattr *nla)
  */
 int nla_ok(const struct nlattr *nla, int remaining)
 {
+	_NL_STATIC_ASSERT(sizeof(*nla) == NLA_HDRLEN);
+
 	return remaining >= (int) sizeof(*nla) &&
 	       nla->nla_len >= sizeof(*nla) &&
 	       nla->nla_len <= remaining;
@@ -177,6 +184,8 @@ static uint16_t nla_attr_minlen[NLA_TYPE_MAX+1] = {
 	[NLA_U64]	= sizeof(uint64_t),
 	[NLA_STRING]	= 1,
 	[NLA_FLAG]	= 0,
+	[NLA_SINT]	= sizeof(uint32_t),
+	[NLA_UINT]	= sizeof(uint32_t),
 };
 
 static int validate_nla(const struct nlattr *nla, int maxtype,
@@ -199,7 +208,7 @@ static int validate_nla(const struct nlattr *nla, int maxtype,
 	else if (pt->type != NLA_UNSPEC)
 		minlen = nla_attr_minlen[pt->type];
 
-	if (nla_len(nla) < minlen)
+	if (_nla_len(nla) < minlen)
 		return -NLE_RANGE;
 
 	if (pt->maxlen && nla_len(nla) > pt->maxlen)
@@ -349,10 +358,13 @@ int nla_memcpy(void *dest, const struct nlattr *src, int count)
 
 	if (!src)
 		return 0;
-	
-	minlen = min_t(int, count, nla_len(src));
-	memcpy(dest, nla_data(src), minlen);
 
+	minlen = _NL_MIN(count, nla_len(src));
+
+	if (minlen <= 0)
+		return 0;
+
+	memcpy(dest, nla_data(src), minlen);
 	return minlen;
 }
 
@@ -449,14 +461,14 @@ int nla_strcmp(const struct nlattr *nla, const char *str)
 struct nlattr *nla_reserve(struct nl_msg *msg, int attrtype, int attrlen)
 {
 	struct nlattr *nla;
-	int tlen;
+	size_t tlen;
 
 	if (attrlen < 0)
 		return NULL;
 
 	tlen = NLMSG_ALIGN(msg->nm_nlh->nlmsg_len) + nla_total_size(attrlen);
 
-	if (tlen > msg->nm_size)
+	if (tlen > msg->nm_size || tlen > UINT32_MAX)
 		return NULL;
 
 	nla = (struct nlattr *) nlmsg_tail(msg->nm_nlh);
@@ -541,6 +553,9 @@ int nla_put_data(struct nl_msg *msg, int attrtype, const struct nl_data *data)
  */
 int nla_put_addr(struct nl_msg *msg, int attrtype, struct nl_addr *addr)
 {
+	if (nl_addr_get_len(addr) == 0)
+		return -NLE_INVAL;
+
 	return nla_put(msg, attrtype, nl_addr_get_len(addr),
 		       nl_addr_get_binary_addr(addr));
 }
@@ -725,7 +740,7 @@ int64_t nla_get_s64(const struct nlattr *nla)
 {
 	int64_t tmp = 0;
 
-	if (nla && nla_len(nla) >= sizeof(tmp))
+	if (nla && _nla_len(nla) >= sizeof(tmp))
 		memcpy(&tmp, nla_data(nla), sizeof(tmp));
 
 	return tmp;
@@ -755,12 +770,76 @@ uint64_t nla_get_u64(const struct nlattr *nla)
 {
 	uint64_t tmp = 0;
 
-	if (nla && nla_len(nla) >= sizeof(tmp))
+	if (nla && _nla_len(nla) >= sizeof(tmp))
 		memcpy(&tmp, nla_data(nla), sizeof(tmp));
 
 	return tmp;
 }
 
+/**
+ * Add variable-length signed integer attribute to netlink message.
+ * @arg msg             Netlink message.
+ * @arg attrtype        Attribute type.
+ * @arg value           Numeric value to store as payload.
+ *
+ * @see nla_put
+ * @return 0 on success or a negative error code.
+ */
+int nla_put_sint(struct nl_msg *msg, int attrtype, int64_t value)
+{
+	int64_t tmp64 = value;
+	int32_t tmp32 = value;
+
+	if (tmp64 == tmp32)
+		return nla_put_s32(msg, attrtype, tmp32);
+	return nla_put(msg, attrtype, sizeof(int64_t), &value);
+}
+
+/**
+ * Return payload of variable-length signed integer attribute
+ * @arg nla		variable-length signed integer attribute
+ *
+ * @return Payload as 64 bit integer.
+ */
+int64_t nla_get_sint(const struct nlattr *nla)
+{
+	if (nla && _nla_len(nla) == sizeof(int32_t))
+		return nla_get_s32(nla);
+	return nla_get_s64(nla);
+}
+
+/**
+ * Add variable-length unsigned integer attribute to netlink message.
+ * @arg msg             Netlink message.
+ * @arg attrtype        Attribute type.
+ * @arg value           Numeric value to store as payload.
+ *
+ * @see nla_put
+ * @return 0 on success or a negative error code.
+ */
+int nla_put_uint(struct nl_msg *msg, int attrtype, uint64_t value)
+{
+	uint64_t tmp64 = value;
+	uint32_t tmp32 = value;
+
+	if (tmp64 == tmp32)
+		return nla_put_u32(msg, attrtype, tmp32);
+	return nla_put(msg, attrtype, sizeof(uint64_t), &value);
+}
+
+/**
+ * Return payload of variable-length unsigned integer attribute
+ * @arg nla		variable-length unsigned integer attribute
+ *
+ * @return Payload as 64 bit integer.
+ */
+uint64_t nla_get_uint(const struct nlattr *nla)
+{
+	if (nla && _nla_len(nla) == sizeof(uint32_t))
+		return nla_get_u32(nla);
+	return nla_get_u64(nla);
+}
+
 /** @} */
 
 /**
@@ -988,6 +1067,15 @@ void nla_nest_cancel(struct nl_msg *msg, const struct nlattr *attr)
 {
 	ssize_t len;
 
+	if (!attr) {
+		/* For robustness, allow a NULL attr to do nothing. NULL is also
+		 * what nla_nest_start() when out of buffer space.
+		 *
+		 * Warning, before libnl-3.8, the function did not accept NULL!
+		 * If you care, catch NULL yourself. */
+		return;
+	}
+
 	len = (char *) nlmsg_tail(msg->nm_nlh) - (char *) attr;
 	if (len < 0)
 		BUG();
