@@ -1,15 +1,10 @@
-
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * src/lib/pfifo.c     	pfifo module for CLI lib
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2010-2011 Thomas Graf <tgraf@suug.ch>
  */
 
+#include "nl-default.h"
+
 #include <netlink/cli/utils.h>
 #include <netlink/cli/tc.h>
 #include <netlink/route/qdisc/fifo.h>
@@ -66,12 +61,12 @@ static struct nl_cli_tc_module pfifo_module =
 	.tm_parse_argv		= pfifo_parse_argv,
 };
 
-static void __init pfifo_init(void)
+static void _nl_init pfifo_init(void)
 {
 	nl_cli_tc_register(&pfifo_module);
 }
 
-static void __exit pfifo_exit(void)
+static void _nl_exit pfifo_exit(void)
 {
 	nl_cli_tc_unregister(&pfifo_module);
 }
