@@ -3,7 +3,8 @@
  * Copyright (c) 2008 Thomas Graf <tgraf@suug.ch>
  */
 
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
 #include <netlink/netlink.h>
 
 static const char *errmsg[NLE_MAX+1] = {
@@ -42,6 +43,8 @@ static const char *errmsg[NLE_MAX+1] = {
 [NLE_IMMUTABLE]		= "Immutable attribute",
 [NLE_DUMP_INTR]		= "Dump inconsistency detected, interrupted",
 [NLE_ATTRSIZE]		= "Attribute max length exceeded",
+[NLE_HOSTUNREACH]	= "Host is unreachable",
+[NLE_NETDOWN]		= "Network is down",
 };
 
 /**
@@ -103,9 +106,10 @@ int nl_syserr2nlerr(int error)
 	case EBUSY:		return NLE_BUSY;
 	case ERANGE:		return NLE_RANGE;
 	case ENODEV:		return NLE_NODEV;
+	case EHOSTUNREACH:	return NLE_HOSTUNREACH;
+	case ENETDOWN:		return NLE_NETDOWN;
 	default:		return NLE_FAILURE;
 	}
 }
 
 /** @} */
-
