@@ -19,13 +19,16 @@
  * ~~~~
  */
 
-#include <netlink-private/netlink.h>
-#include <netlink-private/utils.h>
+#include "nl-default.h"
+
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/msg.h>
 #include <netlink/handlers.h>
 
+#include "nl-core.h"
+#include "nl-priv-dynamic-core/nl-core.h"
+
 static void print_header_content(FILE *ofd, struct nlmsghdr *n)
 {
 	char flags[128];
