@@ -13,20 +13,22 @@
  * @{
  */
 
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
 #include <netlink/netlink.h>
-#include <netlink-private/route/link/api.h>
+
+#include "link-api.h"
 
 static struct rtnl_link_info_ops dummy_info_ops = {
 	.io_name		= "dummy",
 };
 
-static void __init dummy_init(void)
+static void _nl_init dummy_init(void)
 {
 	rtnl_link_register_info(&dummy_info_ops);
 }
 
-static void __exit dummy_exit(void)
+static void _nl_exit dummy_exit(void)
 {
 	rtnl_link_unregister_info(&dummy_info_ops);
 }
