@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/link/ipgre.c        IPGRE Link Info
- *
- *      This library is free software; you can redistribute it and/or
- *      modify it under the terms of the GNU Lesser General Public
- *      License as published by the Free Software Foundation version 2.1
- *      of the License.
- *
  * Copyright (c) 2014 Susant Sahani <susant@redhat.com>
  */
 
@@ -22,15 +16,20 @@
  * @{
  */
 
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
+#include <linux/if_tunnel.h>
+
 #include <netlink/netlink.h>
 #include <netlink/attr.h>
 #include <netlink/utils.h>
 #include <netlink/object.h>
 #include <netlink/route/rtnl.h>
 #include <netlink/route/link/ipgre.h>
-#include <netlink-private/route/link/api.h>
-#include <linux/if_tunnel.h>
+
+#include "nl-route.h"
+#include "link-api.h"
+#include "nl-aux-route/nl-route.h"
 
 #define IPGRE_ATTR_LINK          (1 << 0)
 #define IPGRE_ATTR_IFLAGS        (1 << 1)
@@ -42,6 +41,7 @@
 #define IPGRE_ATTR_TTL           (1 << 7)
 #define IPGRE_ATTR_TOS           (1 << 8)
 #define IPGRE_ATTR_PMTUDISC      (1 << 9)
+#define IPGRE_ATTR_FWMARK        (1 << 10)
 
 struct ipgre_info
 {
@@ -55,6 +55,7 @@ struct ipgre_info
 	uint32_t   link;
 	uint32_t   local;
 	uint32_t   remote;
+	uint32_t   fwmark;
 	uint32_t   ipgre_mask;
 };
 
@@ -69,6 +70,7 @@ static  struct nla_policy ipgre_policy[IFLA_GRE_MAX + 1] = {
 	[IFLA_GRE_TTL]      = { .type = NLA_U8 },
 	[IFLA_GRE_TOS]      = { .type = NLA_U8 },
 	[IFLA_GRE_PMTUDISC] = { .type = NLA_U8 },
+	[IFLA_GRE_FWMARK]   = { .type = NLA_U32 },
 };
 
 static int ipgre_alloc(struct rtnl_link *link)
@@ -91,7 +93,7 @@ static int ipgre_alloc(struct rtnl_link *link)
 static int ipgre_parse(struct rtnl_link *link, struct nlattr *data,
 		       struct nlattr *xstats)
 {
-	struct nlattr *tb[IFLA_IPTUN_MAX + 1];
+	struct nlattr *tb[IFLA_GRE_MAX + 1];
 	struct ipgre_info *ipgre;
 	int err;
 
@@ -157,6 +159,11 @@ static int ipgre_parse(struct rtnl_link *link, struct nlattr *data,
 		ipgre->ipgre_mask |= IPGRE_ATTR_PMTUDISC;
 	}
 
+	if (tb[IFLA_GRE_FWMARK]) {
+		ipgre->fwmark = nla_get_u32(tb[IFLA_GRE_FWMARK]);
+		ipgre->ipgre_mask |= IPGRE_ATTR_FWMARK;
+	}
+
 	err = 0;
 
 errout:
@@ -175,10 +182,10 @@ static int ipgre_put_attrs(struct nl_msg *msg, struct rtnl_link *link)
 	if (ipgre->ipgre_mask & IPGRE_ATTR_LINK)
 		NLA_PUT_U32(msg, IFLA_GRE_LINK, ipgre->link);
 
-	if (ipgre->ipgre_mask & IFLA_GRE_IFLAGS)
+	if (ipgre->ipgre_mask & IPGRE_ATTR_IFLAGS)
 		NLA_PUT_U16(msg, IFLA_GRE_IFLAGS, ipgre->iflags);
 
-	if (ipgre->ipgre_mask & IFLA_GRE_OFLAGS)
+	if (ipgre->ipgre_mask & IPGRE_ATTR_OFLAGS)
 		NLA_PUT_U16(msg, IFLA_GRE_OFLAGS, ipgre->oflags);
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_IKEY)
@@ -202,6 +209,9 @@ static int ipgre_put_attrs(struct nl_msg *msg, struct rtnl_link *link)
 	if (ipgre->ipgre_mask & IPGRE_ATTR_PMTUDISC)
 		NLA_PUT_U8(msg, IFLA_GRE_PMTUDISC, ipgre->pmtudisc);
 
+	if (ipgre->ipgre_mask & IPGRE_ATTR_FWMARK)
+		NLA_PUT_U32(msg, IFLA_GRE_FWMARK, ipgre->fwmark);
+
 	nla_nest_end(msg, data);
 
 nla_put_failure:
@@ -225,10 +235,12 @@ static void ipgre_dump_line(struct rtnl_link *link, struct nl_dump_params *p)
 static void ipgre_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 {
 	struct ipgre_info *ipgre = link->l_info;
-	char *name, addr[INET_ADDRSTRLEN];
-	struct rtnl_link *parent;
+	char addr[INET_ADDRSTRLEN];
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_LINK) {
+		_nl_auto_rtnl_link struct rtnl_link *parent = NULL;
+		char *name;
+
 		nl_dump(p, "      link ");
 
 		name = NULL;
@@ -254,7 +266,7 @@ static void ipgre_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_IKEY) {
 		nl_dump(p, "    ikey   ");
-		nl_dump_line(p, "%x\n",ipgre->ikey);
+		nl_dump_line(p, "%x\n", ipgre->ikey);
 	}
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_OKEY) {
@@ -264,18 +276,14 @@ static void ipgre_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_LOCAL) {
 		nl_dump(p, "      local ");
-		if(inet_ntop(AF_INET, &ipgre->local, addr, sizeof(addr)))
-			nl_dump_line(p, "%s\n", addr);
-		else
-			nl_dump_line(p, "%#x\n", ntohs(ipgre->local));
+		_nl_inet_ntop4(ipgre->local, addr);
+		nl_dump_line(p, "%s\n", addr);
 	}
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_REMOTE) {
 		nl_dump(p, "      remote ");
-		if(inet_ntop(AF_INET, &ipgre->remote, addr, sizeof(addr)))
-			nl_dump_line(p, "%s\n", addr);
-		else
-			nl_dump_line(p, "%#x\n", ntohs(ipgre->remote));
+		_nl_inet_ntop4(ipgre->remote, addr);
+		nl_dump_line(p, "%s\n", addr);
 	}
 
 	if (ipgre->ipgre_mask & IPGRE_ATTR_TTL) {
@@ -292,6 +300,11 @@ static void ipgre_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 		nl_dump(p, "      pmtudisc ");
 		nl_dump_line(p, "enabled (%#x)\n", ipgre->pmtudisc);
 	}
+
+	if (ipgre->ipgre_mask & IPGRE_ATTR_FWMARK) {
+		nl_dump(p, "      fwmark ");
+		nl_dump_line(p, "%x\n", ipgre->fwmark);
+	}
 }
 
 static int ipgre_clone(struct rtnl_link *dst, struct rtnl_link *src)
@@ -829,13 +842,53 @@ uint8_t rtnl_link_get_pmtudisc(struct rtnl_link *link)
 	return rtnl_link_ipgre_get_pmtudisc (link);
 }
 
-static void __init ipgre_init(void)
+/**
+ * Set IPGRE tunnel fwmark
+ * @arg link            Link object
+ * @arg fwmark          fwmark
+ *
+ * @return 0 on success or a negative error code
+ */
+int rtnl_link_ipgre_set_fwmark(struct rtnl_link *link, uint32_t fwmark)
+{
+	struct ipgre_info *ipgre = link->l_info;
+
+	IS_IPGRE_LINK_ASSERT(link);
+
+	ipgre->fwmark = fwmark;
+	ipgre->ipgre_mask |= IPGRE_ATTR_FWMARK;
+
+	return 0;
+}
+
+/**
+ * Get IPGRE tunnel fwmark
+ * @arg link            Link object
+ * @arg fwmark          addr to fill in with the fwmark
+ *
+ * @return 0 on success or a negative error code
+ */
+int rtnl_link_ipgre_get_fwmark(struct rtnl_link *link, uint32_t *fwmark)
+{
+	struct ipgre_info *ipgre = link->l_info;
+
+	IS_IPGRE_LINK_ASSERT(link);
+
+	if (!(ipgre->ipgre_mask & IPGRE_ATTR_FWMARK))
+		return -NLE_NOATTR;
+
+	*fwmark = ipgre->fwmark;
+
+	return 0;
+}
+
+static void _nl_init ipgre_init(void)
 {
 	rtnl_link_register_info(&ipgre_info_ops);
 	rtnl_link_register_info(&ipgretap_info_ops);
 }
 
-static void __exit ipgre_exit(void)
+static void _nl_exit ipgre_exit(void)
 {
 	rtnl_link_unregister_info(&ipgre_info_ops);
 	rtnl_link_unregister_info(&ipgretap_info_ops);
