@@ -15,15 +15,15 @@
  * accordingly via nl_cli_fatal().
  */
 
-#include <netlink/cli/utils.h>
+#include "nl-default.h"
+
 #include <locale.h>
-
-#include "lib/defs.h"
-
 #ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
 #endif
 
+#include <netlink/cli/utils.h>
+
 /**
  * Parse a text based 32 bit unsigned integer argument
  * @arg arg		Integer in text form.
@@ -223,7 +223,7 @@ void nl_cli_load_module(const char *prefix, const char *name)
 	char path[FILENAME_MAX+1];
 
 	snprintf(path, sizeof(path), "%s/%s/%s.so",
-		 PKGLIBDIR, prefix, name);
+		 _NL_PKGLIBDIR, prefix, name);
 
 #ifdef HAVE_DLFCN_H
 	{
