@@ -1,20 +1,16 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * src/nl-link-enslave.c     Enslave a link
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2011 Thomas Graf <tgraf@suug.ch>
  */
 
+#include "nl-default.h"
+
+#include <linux/netlink.h>
+
 #include <netlink/cli/utils.h>
 #include <netlink/cli/link.h>
 #include <netlink/route/link/bonding.h>
 
-#include <linux/netlink.h>
-
 int main(int argc, char *argv[])
 {
 	struct nl_sock *sock;
