@@ -1,20 +1,16 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * src/nl-route-delete.c     Delete Routes
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2009 Thomas Graf <tgraf@suug.ch>
  */
 
+#include "nl-default.h"
+
+#include <linux/netlink.h>
+
 #include <netlink/cli/utils.h>
 #include <netlink/cli/route.h>
 #include <netlink/cli/link.h>
 
-#include <linux/netlink.h>
-
 static int interactive = 0, default_yes = 0, quiet = 0;
 static int deleted = 0;
 static struct nl_sock *sock;
