@@ -3,6 +3,8 @@
  * Copyright (c) 2013 Thomas Graf <tgraf@suug.ch>
  */
 
+#include "nl-default.h"
+
 #include <check.h>
 
 #include "cksuite-all.h"
@@ -25,6 +27,9 @@ int main(int argc, char *argv[])
 	srunner_add_suite(runner, make_nl_attr_suite());
 	srunner_add_suite(runner, make_nl_ematch_tree_clone_suite());
 	srunner_add_suite(runner, make_nl_netns_suite());
+	srunner_add_suite(runner, make_nl_ip6_tnl_suite());
+	srunner_add_suite(runner, make_nl_route_nh_suite());
+	srunner_add_suite(runner, make_nl_route_nexthop_suite());
 
 	srunner_run_all(runner, CK_ENV);
 
