@@ -429,6 +429,9 @@ char **_nltst_strtokv(const char *str);
 
 void nltst_netns_fixture_setup(void);
 void nltst_netns_fixture_teardown(void);
+bool nltst_netns_fixture_is_unshared(void);
+
+bool _nltst_skip_no_netns(void);
 
 struct nltst_netns;
 
@@ -539,6 +542,20 @@ void _nltst_delete_link(struct nl_sock *sk, const char *ifname);
 void _nltst_get_link(struct nl_sock *sk, const char *ifname, int *out_ifindex,
 		     struct rtnl_link **out_link);
 
+void _nltst_add_dummy_and_up(struct nl_sock *sk, const char *ifname,
+			     int *out_ifindex);
+void _nltst_add_dummy_v4_with_addr(struct nl_sock *sk, const char *ifname,
+				   int *out_ifindex, const char *ip,
+				   int prefixlen);
+void _nltst_add_dummy_v6_with_addr(struct nl_sock *sk, const char *ifname,
+				   int *out_ifindex, const char *ip,
+				   int prefixlen);
+
+void _nltst_addr4_add(struct nl_sock *sk, int ifindex, const char *ip,
+		      int prefixlen);
+void _nltst_addr6_add(struct nl_sock *sk, int ifindex, const char *ip,
+		      int prefixlen);
+
 void _nltst_assert_route_list(struct nl_object *const *objs, ssize_t len,
 			      const char *const *expected_routes);
 
@@ -549,4 +566,13 @@ void _nltst_assert_route_cache_v(struct nl_cache *cache,
 	_nltst_assert_route_cache_v(cache,    \
 				    ((const char *const[200]){ __VA_ARGS__ }))
 
+static void _nltst_delete_link_no_sock(const char *ifname)
+{
+	if (ifname)
+		_nltst_delete_link(NULL, ifname);
+}
+#define _nltst_auto_delete_link _nl_auto(_nltst_auto_delete_link_fcn)
+_NL_AUTO_DEFINE_FCN_TYPED0(const char *, _nltst_auto_delete_link_fcn,
+			   _nltst_delete_link_no_sock);
+
 #endif /* __NL_TEST_UTIL_H__ */
